% the ManOpt tool needs to be installed via running manopt/importmanopt.m
% Fig3. Convergence Rate of Normalized Objective Function
warning off
clear;clc; 
randn('seed',1); rand('seed',1);
addpath('./fun');

%% parameter setting: K = 200, r = 15
rank_set =3; K=40;
d = 3;
rank_len = length(rank_set);
obs_num = round(d* rank_set *K*log(K));

params.verbosity = 0;
params.tolgradnorm = 1e-16;
params.maxiter =20;
params.sigma=0.18;

options1=params; options1.mu=1;options1.Delta0 =1;options1.Delta_bar =2;
options1.rho_prime  = 0;

options3.maxiter = 20;options3.tol=5*1e-6;options3.verbosity=1;options3.sigma = 0.18;
options3.mu=1;options3.lbtol =1e-3;options3.deltau = 2;


options4 = struct();
options4.iterations = 10000; 
options4.stepMax    = 10000;
options4.stepMin    = 1e-4;
options4.optTol     = 1e-4;
options4.stepMax    = 1e9;
options4.verbosity    = 0;
options4.sigma = params.sigma;


testnum=1;



results1 = zeros(rank_len,testnum);
results3 = zeros(rank_len,testnum);
results4 = zeros(rank_len,testnum);

reErr1 = zeros(rank_len,testnum);
reErr3 = zeros(rank_len,testnum);
reErr4 = zeros(rank_len,testnum);

t1 = zeros(rank_len,testnum);
t3 = zeros(rank_len,testnum);
t4 = zeros(rank_len,testnum);

alltime=tic;

for test = 1:testnum
    for j = 1:rank_len
        r = rank_set(j);
        [X,Y,ranking] = gen_data(K,K,r);
        [Y_obs,Y_test] = gen_obs(X,Y,obs_num,0);
        XL = randn(K,r); XR = randn(K,r);
        Xo = XL*XR';
        scale = max(abs(Xo(:)));
        XL = XL/sqrt(scale)*sqrt(0.95); XR=XR/sqrt(scale)*sqrt(0.95);
%% log sum exp  regulization term with rank constraint / trust region algorithm
        ts=tic;
        per = obs_num/(r*K*log(K));
        lambda =r*obs_num/(sqrt(K)*per);
        [Xcg1,~,info1] = ranking_fixedrankTR_logsumexp(r, Y_obs, K ,K,{XL,XR},X,params,options1,lambda);
        X1 = Xcg1.L*Xcg1.R';
        [results1(j,test),reErr1(j,test)] = predicting_error(X1,X,params);
        t1(j,test) = toc(ts);
        cost1 = [info1.cost]';
        cost1 = cost1/cost1(end);
        fprintf('TR finishes\n');
        %% log sum exp  regulization term with rank constraint / trust region algorithm
        ts=tic;
        per = obs_num/(r*K*log(K));
        lambda =r*obs_num/(sqrt(K)*per);
        [Xcg1,~,info2] = ranking_fixedrankTR_logsumexp_(r, Y_obs, K ,K,{XL,XR},X,params,options1,lambda);
        X2 = Xcg1.L*Xcg1.R';
        [results2(j,test),reErr2(j,test)] = predicting_error(X2,X,params);
        t12(j,test) = toc(ts);
        cost2 = [info2.cost]';
        cost2 = cost2/cost2(end);
        fprintf('TR finishes\n');
%% BFGD_logbarrier
        ts=tic;
        [X3,info3,infof3] = BFGD_logbarrier(r, Y_obs, K ,K,{XL,XR},X,params,options3);
        [results3(j,test),reErr3(j,test)] = predicting_error(X3,X,params);
        t3(j,test) = toc(ts);
        cost3 = infof3.deltacost;
        cost3 = cost3/cost3(end);
        fprintf('BFGD_logbarrier finishes\n');
%%  SPG
        ts=tic;
        [X4,info4,infof4] = ranking_SPG(r, Y_obs, K ,K,{XL,XR},X,params,options4);
        [results4(j,test),reErr4(j,test)] = predicting_error(X4,X,params);
        t4(j,test) = toc(ts);
        cost4 = [info4.cost]';
        cost4 = cost4/cost4(end);
        fprintf('SPG finishes\n');
        
     end
end
% filename = 'converge_rate_';
% save([filename '.mat']);

aver1 = mean(results1,2);
aver3 = mean(results3,2);
aver4 = mean(results4,2);

aver_re1 = mean(reErr1,2);
aver_re3 = mean(reErr3,2);
aver_re4 = mean(reErr4,2);

iter = 1:20;
figure,plot(iter,cost1(iter),'LineWidth',1.2);hold on
plot(iter,cost2(iter),'LineWidth',1.2);
% 
% plot(iter,cost3(iter),'LineWidth',1.2);
% plot(iter,cost4(iter),'LineWidth',1.2);
hold off;
legend('PRTPS','RSD');
xlabel('iterstion');ylabel('objective function');
set(gca,'Fontsize',12);
axis tight;
grid on